/*
    Copyright 2010 Marco A. Montes de Oca, Dogan Aydin and Thomas Stuetzle

    This file is part of IPSOLS.

    IPSOLS is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    IPSOLS is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with IPSOLS.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "rng.h"
#include "problem.h"
#include "bohachevsky.h"

#include <iostream>
#include <cstdlib>
#include <cmath>

using namespace std;

Bohachevsky::Bohachevsky(Configuration* config):Problem(config){
	optimum = 0.0;
}

Bohachevsky::~Bohachevsky(){
}

long double Bohachevsky::getFunctionValue(const double* x) {
	long double result;

	if(evaluations < maxEvaluations){

		result = bohachevskyFunction(dimension, x);

		//Update number of function evaluations
		evaluations++;

		oldResult  = result;
		
		//Update and print best solution value
		if( bestSolutionValue > result){
			bestSolutionValue = result;
			setBestSoFarSolution(x);
			printProgress();
		}

		return result;
	}
	else
		return oldResult;

}

long double Bohachevsky::bohachevskyFunction(int dim, const double* x){
    long double sum = 0.0;
    int i;
    const double pi = 3.141592653589793;
    double currentGen;
    double nextGen;

    currentGen = x[0] - bohachevsky[0];

    for (i = 1; i < dim; i++)
    {
        nextGen = x[i] - bohachevsky[i];
        sum += currentGen * currentGen + 2.0 * nextGen * nextGen;
        sum += -0.3 * cos(3.0 * pi * currentGen) -0.4 * cos(4.0 * pi * nextGen) + 0.7;
        currentGen = nextGen;
    }

    return sum;
}

long double Bohachevsky::bohachevskyNoShiftFunction(int dim, const double* x){
    long double sum = 0.0;
    int i;
    double currentGen;
    double nextGen;

    currentGen = x[0];

    for (i = 1; i < dim; i++)
    {
        nextGen = x[i];
        sum += currentGen * currentGen + 2.0 * nextGen * nextGen;
        sum += -0.3 * cos(3.0 * PI * currentGen) -0.4 * cos(4.0 * PI * nextGen) + 0.7;
        currentGen = nextGen;
    }

    return sum;
}
