/*
    Copyright 2010 Marco A. Montes de Oca, Dogan Aydin and Thomas Stuetzle

    This file is part of IPSOLS.

    IPSOLS is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    IPSOLS is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with IPSOLS.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef CONFIGURATION_H
#define CONFIGURATION_H

#include <vector>
#include <cmath>

//problems
#define	SHIFTEDSPHERE 		0
#define 	SCHWEFEL221			1
#define 	SHIFTEDROSENBROCK 	2
#define 	SHIFTEDRASTRIGIN 		3
#define 	SHIFTEDGRIEWANK 		4
#define 	SHIFTEDACKLEY 			5
#define 	SCHWEFEL222 			6
#define 	SCHWEFEL12 			7
#define 	EXTENDEDF10 			8
#define 	BOHACHEVSKY 			9
#define 	SCHAFFER 				10
#define	H1 						11
#define 	H2						12
#define 	H3						13
#define 	H4						14
#define 	H7						15
#define 	H8						16
#define 	H9						17
#define 	H10						18

//local search algorithms
#define 	NO_LOCAL_SEARCH		0
#define     POWELL          1

//Topologies
#define FC_TOPOLOGY	0
#define RING_TOPOLOGY	1

#define PI acos(-1)
#define E  exp(1)

#define EPSILON 1E-15

#define MAX_DIMENSION 1000


class Configuration {

      private:

	//general parameters
	unsigned long rngSeed;
	unsigned int maxFES;

	//problem parameters
	unsigned int problemID;
	unsigned int problemDimension;
	double minInitRange;
	double maxInitRange;

        //local search parameters
	unsigned int localSearchID;
	unsigned int lsIters;
	double ftol;

	//swarm parameters
	unsigned int initialPopulationSize;
	unsigned int maxPopulationSize;
 	unsigned int growthPeriod;
	unsigned topologyType;
	double accCoef1, accCoef2;
	double inertiaW;

	//Extra variables
	double startTime;
	unsigned int currentPopulationSize;
	unsigned int maxRepeatedFailures;
	unsigned int maxStagnantIterations;

      public:

	Configuration(int argc,char** argv);
	~Configuration();

	unsigned long getRNGSeed();
	unsigned int getMaxFES();

	unsigned int getProblemID();
	unsigned int getProblemDimension();
	double getMinInitRange();
	double getMaxInitRange();

	unsigned int getLocalSearchID();

	unsigned int getLsIterations();
	double getFTol();

	unsigned int getInitialPopulationSize();
	unsigned int getMaximumPopulationSize();
 	unsigned int getGrowthPeriod();
	unsigned int getTopologyType();

	double getAccCoef1();
	double getAccCoef2();
	double getInertiaWeight();

	void setStartTime(double stime);
	double getStartTime();
	void setCurrentPopulationSize(unsigned int newSize);
	unsigned int getCurrentPopulationSize();
	unsigned int getMaxStagnantIterations();
	unsigned int getMaxRepeatedFailures();

	void print();
};

#endif
