/*
    Copyright 2010 Marco A. Montes de Oca, Dogan Aydin and Thomas Stuetzle

    This file is part of IPSOLS.

    IPSOLS is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    IPSOLS is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with IPSOLS.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "rng.h"
#include "problem.h"
#include "extendedF10.h"

#include <iostream>
#include <cstdlib>
#include <cmath>

using namespace std;

ExtendedF10::ExtendedF10(Configuration* config):Problem(config){
	optimum = 0.0;
}

ExtendedF10::~ExtendedF10(){
}

long double ExtendedF10::getFunctionValue(const double* x) {
	long double result;

	if(evaluations < maxEvaluations){

		result = extendedF10Function(dimension,x);

		//Update number of function evaluations
		evaluations++;

		oldResult  = result;
		
		
		//Update and print best solution value
		if( bestSolutionValue > result){
			bestSolutionValue = result;
			setBestSoFarSolution(x);
			printProgress();
		}

		return result;
	}
	else
		return oldResult;

}

long double ExtendedF10::f_10(double x, double y)
{
     double p, z, t;

     p=(x*x+y*y);

     z=pow(p, 0.25);
     t=sin(50.0*pow(p, 0.1));
     t=t*t+1.0;

     return z*t;
}

long double ExtendedF10::extendedF10Function(int dim,const double* x){
     long double suma=0.0;

     for(int i=0; i<dim-1; i++)
           suma+=f_10(x[i] - extendedf10[i], x[i+1] - extendedf10[i+1]);

     suma+=f_10(x[dim-1] - extendedf10[dim-1], x[0] - extendedf10[0]);

     return suma;
}

long double ExtendedF10::extendedF10NoShiftFunction(int dim,const double* x){
     long double suma=0.0;

     for(int i=0; i<dim-1; i++)
           suma+=f_10(x[i], x[i+1]);

     suma+=f_10(x[dim-1], x[0]);

     return suma;
}
