/*
    Copyright 2010 Marco A. Montes de Oca, Dogan Aydin and Thomas Stuetzle

    This file is part of IPSOLS.

    IPSOLS is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    IPSOLS is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with IPSOLS.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "rng.h"
#include "problem.h"
#include "shiftedsphere.h"
#include "extendedF10.h"
#include "h1.h"

#include <assert.h>
#include <iostream>
#include <cstdlib>
#include <cmath>

using namespace std;

Hybrid1::Hybrid1(Configuration* config):Problem(config){
	optimum = 0.0;
	problem1 = new ExtendedF10(config);
	problem2 = new ShiftedSphere(config);
}

Hybrid1::~Hybrid1(){
	delete problem1;
	delete problem2;
}

long double Hybrid1::getFunctionValue(const double* x) {
	long double result;

	if(evaluations < maxEvaluations){

		result = h1Function(x);

		//Update number of function evaluations
		evaluations++;

		oldResult  = result;

		
		//Update and print best solution value
		if( bestSolutionValue > result){
			bestSolutionValue = result;
			setBestSoFarSolution(x);
			printProgress();
		}

		return result;
	}
	else
		return oldResult;

}

long double Hybrid1::h1Function(const double* x){
    double part1[dimension], part2[dimension];
    int size1, size2;
    long double f1, f2;
    divideFunctions(x, part1, part2, 0.25, &size1, &size2);

    f1 = problem1->extendedF10NoShiftFunction(size1,part1);
    f2 = problem2->shiftedSphereNOPFunction(size2,part2);
    assert(f1 >= 0);
    assert(f2 >= 0);
    
    return f1 + f2;
}
