/*
    Copyright 2010 Marco A. Montes de Oca, Dogan Aydin and Thomas Stuetzle

    This file is part of IPSOLS.

    IPSOLS is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    IPSOLS is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with IPSOLS.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef LOCALSEARCHPROBLEMS_H_INCLUDED
#define LOCALSEARCHPROBLEMS_H_INCLUDED

#include <math.h>
#include <algorithm>

#define abss(a)     (a<0 ? (-a) : a)

extern "C" {


     // dim: dimension, x: values, func_data: additional data for printing and control 
     typedef long double (*obj_func)(int n, const double *x, long double *func_data);
     long double shiftedSphereFunction(int dim, const double* x, long double *my_func_data);
     long double shiftedRastriginFunction(int dim, const double* x, long double *my_func_data);
     long double shiftedRosenbrockFunction(int dim, const double* x, long double *my_func_data);
     long double shiftedGriewankFunction(int dim, const double* x, long double *my_func_data);
     long double shiftedAckleyFunction(int dim,const double* x, long double *my_func_data);
     long double schwefel222Function(int dim, const double* x, long double *my_func_data);
     long double schwefel221Function(int dim, const double* x, long double *my_func_data);
     long double schwefel12Function(int dim, const double* x, long double *my_func_data);
     long double schafferFunction(int dim, const double* x, long double *my_func_data);
     long double h10Function(int dim, const double* x, long double *my_func_data);
     long double h9Function(int dim, const double* x, long double *my_func_data);
     long double h8Function(int dim, const double* x, long double *my_func_data);
     long double h7Function(int dim, const double* x, long double *my_func_data);
     long double h4Function(int dim, const double* x, long double *my_func_data);
     long double h3Function(int dim, const double* x, long double *my_func_data);
     long double h2Function(int dim, const double* x, long double *my_func_data);
     long double h1Function(int dim, const double* x, long double *my_func_data);
     long double h7Function(int dim, const double* x, long double *my_func_data);
     long double extendedF10Function(int dim,const double* x, long double *my_func_data);
     long double extendedF10NoShiftFunction(int dim,const double* x, long double *my_func_data);
     long double bohachevskyFunction(int dim, const double* x,  long double *my_func_data);
     long double bohachevskyNoShiftFunction(int dim, const double* x,   long double *my_func_data);

     void printProgress(long double* func_data, long double result);
    }

#endif // LOCALSEARCHPROBLEMS_H_INCLUDED
