/*
    Copyright 2010 Marco A. Montes de Oca, Dogan Aydin and Thomas Stuetzle

    This file is part of IPSOLS.

    IPSOLS is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    IPSOLS is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with IPSOLS.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef PARTICLE_H
#define PARTICLE_H

#include "rng.h"
#include "configuration.h"
#include "problem.h"



#include <vector>

using namespace std;

class Particle {

    protected:
        Configuration* config;
        Problem* problem;
        int id;

        double* currentPosition;
        double* currentVelocity;
        double* bestPosition;
        long double currentValue;
        long double bestValue;

        bool toimprove;
	unsigned int failedAttempts;


        std::vector<int> neighbors;

    public:

        Particle(Configuration* config,Problem* problem, int identifier);
	Particle(Configuration* config,Problem* problem, int identifier, Particle* bestParticle);
	Particle(Configuration* config,Problem* problem, int identifier, double* bestSoFarSolution,double bestSolutionValue);
        virtual ~Particle();

        int getID();
        void setID(int ID);

        int getDimension();

	void resetTopology();

        std::vector<int> getNeighbors();
        unsigned int getNeighborhoodSize();

        void addNeighbor(int neighborID);
        void deleteNeighbors();

        void evaluateCurrentPosition();

        const double* getCurrentPosition();
        void setCurrentPosition(double* newPosition);

        const double* getCurrentVelocity();
        void setCurrentVelocity(double* newVelocity);

        const double* getBestPosition();
        void setBestPosition(double* newBestPosition);
        void updateBestPosition(long double newBest);

        long double getCurrentValue();
        void setCurrentValue(long double newValue);

        long double getBestValue();
        void setBestValue(long double newBestValue);

        bool toImprove();
        void setToImprove(bool val);

	unsigned int getFailedAttempts();
	void incrementFailedAttempts();
	void resetFailedAttempts();
	
	void reset();

        void print();
};


#endif

