/*
    Copyright 2010 Marco A. Montes de Oca, Dogan Aydin and Thomas Stuetzle

    This file is part of IPSOLS.

    IPSOLS is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    IPSOLS is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with IPSOLS.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "rng.h"
#include "problem.h"
#include "schaffer.h"

#include <iostream>
#include <cstdlib>
#include <cmath>

using namespace std;

Schaffer::Schaffer(Configuration* config):Problem(config){
	optimum = 0.0;
}

Schaffer::~Schaffer(){
}

long double Schaffer::getFunctionValue(const double* x) {
	long double result;

	if(evaluations < maxEvaluations){

		result = schafferFunction(x);

		//Update number of function evaluations
		evaluations++;

		oldResult  = result;

		
		//Update and print best solution value
		if( bestSolutionValue > result){
			bestSolutionValue = result;
			setBestSoFarSolution(x);
			printProgress();
		}

		return result;
	}
	else
		return oldResult;

}

long double Schaffer::schafferFunction(const double* x){
    int i;
    long double sum, aux, aux2;
    double currentGen, nextGen;

    sum = 0.0;
    currentGen = x[0] - schaffer[0];
    currentGen = currentGen * currentGen;

    for (i = 1; i < dimension; i++)
    {
        nextGen = x[i] - schaffer[i];
        nextGen = nextGen * nextGen;
        aux = currentGen + nextGen;
        currentGen = nextGen;
        aux2 = sin(50. * pow((double)aux, 0.1));
        sum += pow((double)aux, 0.25) * (aux2 * aux2 + 1.0);
    }

    return sum;
}
