/*
    Copyright 2010 Marco A. Montes de Oca, Dogan Aydin and Thomas Stuetzle

    This file is part of IPSOLS.

    IPSOLS is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    IPSOLS is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with IPSOLS.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "rng.h"
#include "problem.h"
#include "schwefel12.h"

#include <iostream>
#include <cstdlib>
#include <cmath>

using namespace std;

Schwefel12::Schwefel12(Configuration* config):Problem(config){
	optimum = 0.0;
}

Schwefel12::~Schwefel12(){
}

long double Schwefel12::getFunctionValue(const double* x) {
	long double result;

	if(evaluations < maxEvaluations){

		result = schwefel12Function(x);

		//Update number of function evaluations
		evaluations++;

		oldResult  = result;

		
		//Update and print best solution value
		if( bestSolutionValue > result){
			bestSolutionValue = result;
			setBestSoFarSolution(x);
			printProgress();
		}

		return result;
	}
	else
		return oldResult;

}

long double Schwefel12::schwefel12Function(const double* x){
	long double Sum=0.0, Val=0.0;

	for (int i = 0; i < dimension; i++){
        	Val += x[i] - schwefel1_2[i];
	    	Sum += Val * Val;
    	}

	return Sum;
}
