/*
    Copyright 2010 Marco A. Montes de Oca, Dogan Aydin and Thomas Stuetzle

    This file is part of IPSOLS.

    IPSOLS is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    IPSOLS is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with IPSOLS.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "rng.h"
#include "problem.h"
#include "schwefel221.h"

#include <iostream>
#include <cstdlib>
#include <cmath>

using namespace std;

Schwefel221::Schwefel221(Configuration* config):Problem(config){
	optimum = 0.0;
}

Schwefel221::~Schwefel221(){
}

long double Schwefel221::getFunctionValue(const double* x) {
	long double result;

	if(evaluations < maxEvaluations){

		result = schwefel221Function(x);

		//Update number of function evaluations
		evaluations++;

		oldResult  = result;

		
		//Update and print best solution value
		if( bestSolutionValue > result){
			bestSolutionValue = result;
			setBestSoFarSolution(x);
			printProgress();
		}
	
		return result;
	}
	else
		return oldResult;

}

long double Schwefel221::schwefel221Function(const double* x){
    int i;
    long double z;
    long double F = abss(x[0]);
    for(i=1;i<dimension;i++){
    	  z = x[i] - schwefel[i];
        F = max(F , abss(z));
    }
    return F;
}
