/*
    Copyright 2010 Marco A. Montes de Oca, Dogan Aydin and Thomas Stuetzle

    This file is part of IPSOLS.

    IPSOLS is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    IPSOLS is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with IPSOLS.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "rng.h"
#include "problem.h"
#include "shiftedrastrigin.h"

#include <iostream>
#include <cstdlib>
#include <cmath>

using namespace std;

ShiftedRastrigin::ShiftedRastrigin(Configuration* config):Problem(config){
	optimum = 0.0;
}

ShiftedRastrigin::~ShiftedRastrigin(){
}

long double ShiftedRastrigin::getFunctionValue(const double* x) {
	long double result;

	if(evaluations < maxEvaluations){

		result = shiftedRastriginFunction(dimension, x);

		//Update number of function evaluations
		evaluations++;

		oldResult  = result;

		
		//Update and print best solution value
		if( bestSolutionValue > result){
			bestSolutionValue = result;
			setBestSoFarSolution(x);
			printProgress();
		}
	
		return result;
	}
	else
		return oldResult;

}

long double ShiftedRastrigin::shiftedRastriginFunction(int dim, const double* x){
   int i;
    long double z;
    long double F = 0;
    for(i=0;i<dim;i++){
        z = x[i] - rastrigin[i];
        F = F + ( pow(z,2) - 10*cos(2*PI*z) + 10);
    }
    return F;
}


long double ShiftedRastrigin::shiftedRastriginNOPFunction(int dim, const double* x){
   int i;
    long double z;
    long double F = 0;
    for(i=0;i<dim;i++){
        z = x[i] - rastrigin[i];
        F = F + ( pow(z,2) - 10*cos(2*PI*z) + 10);
    }
    return F;
}
