
/*
    Copyright 2010 Marco A. Montes de Oca, Dogan Aydin and Thomas Stuetzle

    This file is part of IPSOLS.

    IPSOLS is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    IPSOLS is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with IPSOLS.  If not, see <http://www.gnu.org/licenses/>.
*/
#include "rng.h"
#include "problem.h"
#include "shiftedrosenbrock.h"

#include <iostream>
#include <cstdlib>
#include <cmath>

using namespace std;

ShiftedRosenbrock::ShiftedRosenbrock(Configuration* config):Problem(config){
	optimum = 0.0;
}

ShiftedRosenbrock::~ShiftedRosenbrock(){
}

long double ShiftedRosenbrock::getFunctionValue(const double* x) {
	long double result;

	if(evaluations < maxEvaluations){

		result = shiftedRosenbrockFunction(dimension,x);

		//Update number of function evaluations
		evaluations++;

		oldResult  = result;
	
		
		//Update and print best solution value
		if( bestSolutionValue > result){
			bestSolutionValue = result;
			setBestSoFarSolution(x);
			printProgress();
		}


		return result;
	}
	else
		return oldResult;

}

long double ShiftedRosenbrock::shiftedRosenbrockFunction(int dim, const double* x){
   int i;
   long double z[dim];
   long double F = 0;

    for(i=0;i<dim;i++) z[i] = x[i] - rosenbrock[i] + 1;

    for(i=0;i<dim-1;i++){
        F = F + 100*( pow((pow(z[i],2)-z[i+1]) , 2) ) + pow((z[i]-1) , 2);
    }
    return F;
}


long double ShiftedRosenbrock::shiftedRosenbrockNOPFunction(int dim, const double* x){
   int i;
   long double z[dim];
   long double F = 0;

    for(i=0;i<dim;i++) z[i] = x[i] - rosenbrock[i] + 1;

    for(i=0;i<dim-1;i++){
        F = F + 100*( pow((pow(z[i],2)-z[i+1]) , 2) ) + pow((z[i]-1) , 2);
    }
    return F;
}
