/*
    Copyright 2010 Marco A. Montes de Oca, Dogan Aydin and Thomas Stuetzle

    This file is part of IPSOLS.

    IPSOLS is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    IPSOLS is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with IPSOLS.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef SWARM_H
#define SWARM_H

#include "rng.h"
#include "configuration.h"
#include "problem.h"
#include "particle.h"

#include <vector>

class Swarm {

      private:
	  Configuration* config;
	  vector<Particle*> particles;
	  Problem* problem;

      public:

	  Swarm(Configuration* config,Problem* problem);
	  Swarm(Configuration* config,Problem* problem,double* bestSoFarSolution,double bestSolutionValue);
	  ~Swarm();

	  int getSize();

	  Particle* getParticle(int id);
	  Particle* getBestNeighborOf(Particle* p);
	  
	  void addParticle(Particle* newParticle);

	  vector<Particle*> getParticles();
	  long double getSolutionValue();
	  const double* getSolution();
	  Particle* getBestParticle();
	  Particle* getRandomParticle();

	  double Edistance(double* a, const double* b);
	  double Dispersion();

	  void print();
};


#endif
